#!/bin/echo Source me: .
#
# Copyright (c) 2002-2004 by Frederick Bruckman
# All rights reserved.

# mk_index.sh
#
# This helper script for "pkgorder" creates all the indexes used by the
# rest of the program. All the manly work of parsing "pkgsrc" into the
# dependencies graph is factored into two shell functions, one per
# generated file. A third shell function generates the transpose and a
# fourth, the topologically sorted list (courtesy of the "tsort" binary).
# The format of the indexes is human readable and self-explanatory.

#
# Environment Variables (optional)
#

if [ -z "$PKGORDER_LIBDIR" ]
then
	PKGORDER_LIBDIR=/usr/local/lib/pkgorder
fi

if [ ! -d "$PKGSRCDIR" ]
then
	PKGSRCDIR=/usr/pkgsrc
fi

if [ -z "$PKGORDERDIR" ]
then
	PKGORDERDIR=~/.pkgorder
fi

# Source some functions.
#
. "${PKGORDER_LIBDIR}"/create_dir2dependencies
. "${PKGORDER_LIBDIR}"/create_dir2pkgname
. "${PKGORDER_LIBDIR}"/sort_with
. "${PKGORDER_LIBDIR}"/subtract_key
. "${PKGORDER_LIBDIR}"/transpose_deps
. "${PKGORDER_LIBDIR}"/tsort_deps

#
# Do it!
#

# Time stamp before beginning. Not used for anything yet.
#
mkdir -m 755 -p "${PKGORDERDIR}"
touch "${PKGORDERDIR}"/time_stamp

# Create initial dependency lists.
#
set -m
create_dir2dependencies "${PKGORDERDIR}"/dir_to_dependencies_tmp &
create_dir2pkgname "${PKGORDERDIR}"/dir_to_pkgname_tmp &
wait
set +m

transpose_deps "${PKGORDERDIR}"/dir_to_dependencies_tmp \
	"${PKGORDERDIR}"/dir_to_dependents_tmp
tsort_deps "${PKGORDERDIR}"/dir_to_dependents_tmp \
	"${PKGORDERDIR}"/sorted_dirs_tmp \

# Identify "singletons", and place first in final lists.
#
subtract_key "${PKGORDERDIR}"/dir_to_pkgname_tmp \
	"${PKGORDERDIR}"/sorted_dirs_tmp | \
	sort -t / > "${PKGORDERDIR}"/sorted_dirs
cat "${PKGORDERDIR}"/sorted_dirs_tmp >> \
	"${PKGORDERDIR}"/sorted_dirs

# Final sort of dependency lists.
#
SORT="sort_with ${PKGORDERDIR}/sorted_dirs"
$SORT "${PKGORDERDIR}"/dir_to_pkgname_tmp \
	> "${PKGORDERDIR}"/dir_to_pkgname
$SORT "${PKGORDERDIR}"/dir_to_dependencies_tmp \
	> "${PKGORDERDIR}"/dir_to_dependencies
$SORT "${PKGORDERDIR}"/dir_to_dependents_tmp \
	> "${PKGORDERDIR}"/dir_to_dependents

# Clean up.
#
rm "${PKGORDERDIR}"/*_tmp
