#!/bin/sh
#
# Copyright (c) 2002-2004 by Frederick Bruckman
# All rights reserved.

# pkgorder.sh
#
# A program for sorting NetBSD packages. See the man page for what it
# actually does. This shell script is highly factored into functions,
# so it can be presented top-down. (There's no way to forward declare
# a shell function in NetBSD's system shell.)

#
# Functions
#

usage () {
	echo 'usage: pkgorder [-c]|[[-DIdir] [package_spec ... ]]'
	exit 1
}

do_env () {
	if [ -z "$PKG_DBDIR" ]
		then export PKG_DBDIR=/var/db/pkg
	fi
	if [ -z "$PKGORDER_LIBDIR" ]
		then PKGORDER_LIBDIR=/usr/local/lib/pkgorder
	fi
	if [ -z "$PKGORDERDIR" ]
		then PKGORDERDIR=~/.pkgorder
	fi
	if [ ! -d "$PKGSRC" ]
		then PKGSRC=/usr/pkgsrc
	fi
}

do_flags () {
	if [ $# -lt 1 ]
		then usage
	fi
	while getopts 'cDIdir' flag
		do case $flag in
			c)
				# As documented, bail if the user tries to do
				# this in conjunction with other operations.
				if [ $# -gt 1 ]
					then usage
				fi
				mk_index
				exit 0
				;;
			D)
				Flag_D=yes
				;;
			I)
				Flag_I=yes
				;;
			d)
				Flag_d=yes
				;;
			i)
				Flag_i=yes
				;;
			r)
				OUTPUT_FILTER='tac'
				;;
			*)
				usage
				;;
		esac
	done
}

# Collect the packages specified on the command line, then sort them.
#
do_args () {
	# Create a temporary file, and dump the initial arguments into it
	# (in canononical form).
	TMP_FILE="$(mktemp -t pkgorder)"
	canonify "$@" > "$TMP_FILE"

	# This is like listing all installed packages on the command line.
	if [ -n "$Flag_i" ]
		then dump_installed >> "$TMP_FILE"
	fi

	# Flag_I only applies to the command line arguments, so do now.
	if [ -s "${TMP_FILE}" -a -n "$Flag_I" ]
	then
		while ! cmp -s "${TMP_FILE}" "${TMP_FILE}".bak
		do
			sort -u "${TMP_FILE}" > "${TMP_FILE}".bak
			fill_out_I "${TMP_FILE}".bak | sort -u \
				> "${TMP_FILE}"
		done
	fi

	# Recurse over the dependencies, until we're sure we got them all.
	if [ -s "${TMP_FILE}" -a \( -n "$Flag_D" -o -n "$Flag_d" \) ]
	then
		while ! cmp -s "${TMP_FILE}" "${TMP_FILE}".bak
		do
			sort -u "${TMP_FILE}" > "${TMP_FILE}".bak
			fill_out_Dd "${TMP_FILE}".bak | sort -u \
				> "${TMP_FILE}"
		done
	fi

	# Now sort the output, and clean up.
	sort_with "${PKGORDERDIR}"/sorted_dirs "${TMP_FILE}" \
	| ${OUTPUT_FILTER:-cat}
	rm -f "${TMP_FILE}" "${TMP_FILE}".bak
}

#
# Do it!
#

do_env

# Source some more functions.
#
. "${PKGORDER_LIBDIR}"/canonify
. "${PKGORDER_LIBDIR}"/dir_to_pkg
. "${PKGORDER_LIBDIR}"/dump_installed
. "${PKGORDER_LIBDIR}"/fill_out_I
. "${PKGORDER_LIBDIR}"/fill_out_Dd
. "${PKGORDER_LIBDIR}"/mk_index
. "${PKGORDER_LIBDIR}"/sort_with
. "${PKGORDER_LIBDIR}"/tac

do_flags "$@"
shift $(($OPTIND - 1))
do_args "$@"
